@@ -332,7 +332,7 @@ module SiSU_Syntax
         and defined? @md.creator \
         and @md.creator
           if dob.is =='heading'
-            dob.obj.gsub!(/^\s*@title\s*$/,@md.title.full) if dob.lv =~/1/                #
+            dob.obj.gsub!(/^\s*@title\s*$/,@md.title.full) if dob.lv =~/1/
             if dob.lv =~/[23]/ \
             and defined? @md.creator.author \
             and @md.creator.author
@@ -432,9 +432,6 @@ module SiSU_Syntax
         dob=fontface(dob)
       elsif dob.is =='code'
         dob.obj.gsub!(/#{Mx[:meta_o]}(\S+?)#{Mx[:meta_c]}\s*/,'@\1: ')
-        dob.obj.gsub!(/([<>])/,'_\1')
-        dob.obj.gsub!(/_<:(\S+?)_>/,'<:\1>') #convert <:\S+> back, clumsy
-        dob.obj.gsub!(/_<(br(?: \/)?)_>/,'<\1>') #convert <br> <br /> back, clumsy
         dob.obj.gsub!(/(^|#{Mx[:gl_c]}|\s)&lt;(br(?: \/)?)&gt;([\s,.]|$)/,'\1<\2>\3') #convert <br> <br /> back, clumsy
         if dob.number_
           codeline=[]
@@ -443,7 +440,7 @@ module SiSU_Syntax
             unless i == 0
               cl.gsub!(Mx[:br_nl],'')
               w=3-ln.to_s.length
-              cl = "#{ln}#{Mx[:nbsp]*w}#{Mx[:vline]}#{Mx[:nbsp]}#{Mx[:nbsp]}#{cl}#{Mx[:br_nl]}"
+              cl = "#{ln}#{Mx[:nbsp]*w}#{Mx[:vline]}#{cl}#{Mx[:br_nl]}"
               ln +=1
             end
             codeline << cl
@@ -451,8 +448,7 @@ module SiSU_Syntax
           codeline= codeline.join("")
           dob.obj=codeline
         else
-          dob.obj.gsub!(/#{Mx[:gr_o]}codeline#{Mx[:gr_c]}/,
-            "\n#{Mx[:nbsp]}#{Mx[:nbsp]}") #temporary fix, prefer: #line.gsub!(/<:codeline>/,"\n")
+          dob.obj.gsub!(/#{Mx[:gr_o]}codeline#{Mx[:gr_c]}/,"\n")
         end
         dob
       else # @\S+?:
