@@ -634,7 +634,6 @@ WOK
       def segtoc
         begin
           @filename_html_segtoc=@file.write_file.html_segtoc
-          @filename_html_index=@file.write_file.html_seg_index
           @data.each do |para|
             para.strip!
             para.gsub!(/<!.+?!>/,'')
@@ -642,19 +641,16 @@ WOK
             para.gsub!(Xx[:html_relative1],@file.path_rel_links.html_seg_1)
             unless para =~/\A\s*\Z/
               @filename_html_segtoc.puts para,"\n"
-              @filename_html_index.puts para,"\n"
             end
           end
         rescue; SiSU_Errors::Info_error.new($!,$@,@md.opt.cmd,@md.fns).error
         ensure
           @filename_html_segtoc.close
-          @filename_html_index.close
-          ## only works properly for directory_structure_by language, fix
-          unless FileTest.file?("#{@file.output_path.html.dir}/index.html")
+          unless FileTest.file?("#{@file.output_path.html_seg.dir}/#{@md.file.base_filename.html_seg_index}")
             pwd_set=Dir.pwd
             idx_lnk=@file.base_filename.html_segtoc
-            mlnk='index.html'
-            Dir.chdir(@file.output_path.html.dir)
+            mlnk=@file.base_filename.html_seg_index
+            Dir.chdir(@file.output_path.html_seg.dir)
             FileUtils::rm_f(mlnk)
             FileUtils::ln_s(idx_lnk, mlnk)
             Dir.chdir(pwd_set)
