@@ -131,7 +131,12 @@ module SiSU_commandline
           if find_flag \
           && y !~ /^-/ \
           && y =~ /\S+/
-            f << y
+            if y !~/\//
+              f << y
+            else
+              find_flag=false
+              puts %{sub-directories "#{y}" cannot be provided for --find or --glob at this time}
+            end
           end
         end
         r=Px[:lng_lst].join('|')
