@@ -109,7 +109,7 @@ module SiSU_XML_munge
     end
     def char_enc #character encode
       def utf8(dob='')
-        if @sys.locale =~/utf-?8/i # instead ucs for utf8 #require 'iconv' ? Iñtërnâtiônàlizætiøn
+        if @sys.locale =~/utf-?8/i # instead ucs for utf8 # String#encode Iñtërnâtiônàlizætiøn
           str=if defined? dob.obj; dob.obj
           elsif dob.class==String; dob
           end
@@ -224,7 +224,7 @@ module SiSU_XML_munge
         end
       end
       def html(dob='')
-        if @sys.locale =~/utf-?8/i # instead ucs for utf8 #require 'iconv' ? Iñtërnâtiônàlizætiøn
+        if @sys.locale =~/utf-?8/i # instead ucs for utf8 # String#encode Iñtërnâtiônàlizætiøn
           dob.obj.gsub!(/ /u,' ')           # space identify
           dob.obj.gsub!(/ /u,' ')           # space identify
         else
